/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/src/net/beandocument/BeanAttribute.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-02-27 15:11:14 +0100 (So, 27 Feb 2005) $
** Revision .........: $LastChangedRevision: 11 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Implementation for attributes in bean context
 * @author Jens Scheffler
 */
public class BeanAttribute extends BeanBaseNode implements Attr {

    /** Name of this attribute */
    private final String name;
    
    /** Value of this attribute */
    private final String value;

    /**
     * Constructs a new Attribute based on the given object
     * @param owner Owner document of this element
     * @param parent Parent element of the new element
     * @param name tag name of the new element
     * @param value Value bean to use for tag content
     */
    protected BeanAttribute(BeanDocument owner, BeanBaseNode parent, String name, String value) {
        super(owner, parent);

        this.name = name;
        this.value = value;
    }

    /**
     * Returns flag if this attribute was specified or generate implicit
     * @return always true in this implementation
     * @see org.w3c.dom.Attr#getSpecified()
     */
    public boolean getSpecified() {
        return true;
    }

    /**
     * Retrieves attribute name
     * @return Name of the Attribute
     * @see org.w3c.dom.Attr#getName()
     */
    public String getName() {
        return name;
    }

    /**
     * Retrieves the value of the attribute
     * @return Value of the attribute
     * @see org.w3c.dom.Attr#getValue()
     */
    public String getValue() {
        return value;
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param value New value to set - ignored by this implementation
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Attr#setValue(java.lang.String)
     */
    public void setValue(String value) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * Gets the parent element where this attribute belongs to
     * @return Parent element
     * @see org.w3c.dom.Attr#getOwnerElement()
     */
    public Element getOwnerElement() {
        return (Element) getParentNode();
    }

    /**
     * Retrieves the node type of the attribute, will always return a constant value
     * @return always Node.ATTRIBUTE_NODE
     * @see org.w3c.dom.Node#getNodeType()
     */
    public short getNodeType() {
        return Node.ATTRIBUTE_NODE;
    }

    /**
     * Returns always false
     * @return false value
     * @see org.w3c.dom.Node#hasAttributes()
     */
    public boolean hasAttributes() {
        return false;
    }

    /**
     * Returns always false
     * @return false value
     * @see org.w3c.dom.Node#hasChildNodes()
     */
    public boolean hasChildNodes() {
        return false;
    }

    /**
     * Retrieves the local attribute name which corresponds with the attribute name itself
     * @return attribute local name
     * @see org.w3c.dom.Node#getLocalName()
     */
    public String getLocalName() {
        return getName();
    }

    /**
     * Retrieves the name of the node, which corresponds with the attribute name
     * @return attribute name
     * @see org.w3c.dom.Node#getNodeName()
     */
    public String getNodeName() {
        return getName();
    }

    /**
     * Retrieves the node value which corresponds with the attribute value
     * @return Attribute value
     * @throws DOMException (never throws this exception)
     * @see org.w3c.dom.Node#getNodeValue()
     */
    public String getNodeValue() throws DOMException {
        return getValue();
    }

    /**
     * Retrieves the attributes of the node. Since the attribute node itself has no attributes, returns null.
     * @return null value
     * @see org.w3c.dom.Node#getAttributes()
     */
    public NamedNodeMap getAttributes() {
        return null;
    }

    /**
     * Retrieves the first child, which is (of course) not existing here.
     * @return null value
     * @see org.w3c.dom.Node#getFirstChild()
     */
    public Node getFirstChild() {
        return null;
    }

    /**
     * Retrieves the last child, which is (of course) not existing here.
     * @return null value
     * @see org.w3c.dom.Node#getLastChild()
     */
    public Node getLastChild() {
        return null;
    }

    /**
     * Retrieves the list of child nodes, which is an empty list in this case.
     * @return empty list of nodes
     * @see org.w3c.dom.Node#getChildNodes()
     */
    public NodeList getChildNodes() {
        return new BeanNodeList();
    }
}
