/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/src/net/beandocument/BeanBaseNode.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-02-27 15:11:14 +0100 (So, 27 Feb 2005) $
** Revision .........: $LastChangedRevision: 11 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Base node implementation for all bean node stuff which is general for the DOM implementation
 * @author Jens Scheffler
 */
public abstract class BeanBaseNode implements Node {

    /** Error message to present when user tries to modify this document implementation content */
    static final String MODIFICATION_ERROR_MESSAGE = 
        "Document is not able to be modified. Please use only read-only!";

    /** Owner document for this node */
    protected BeanDocument owner;

    /** parent element for this element instance */
    protected final BeanBaseNode parent;

    /** Next sibling in element context */
    private Node nextSibling;

    /** Previous sibling in element context */
    private Node previousSibling;
    
    /**
     * Default Constructor
     * @param owner Owner document where this node belongs to
     * @param parent Parent node of the new created node
     */
    protected BeanBaseNode(BeanDocument owner, BeanBaseNode parent) {
        super();

        this.owner = owner;
        this.parent = parent;
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param node Node to add - ignored by this implementation
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Node#appendChild(org.w3c.dom.Node)
     */
    public final Node appendChild(Node node) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param recursive Flag if depending nodes should be cloned recursive - ignored by this implementation
     * @return (This function will never return normally)
     * @throws IllegalArgumentException If the method is called
     * @see org.w3c.dom.Node#cloneNode(boolean)
     */
    public final Node cloneNode(boolean recursive) {
        throw new IllegalArgumentException(MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * Retrieves the namespace of this node - will always be null since namespace support is not implemented
     * @return null value
     * @see org.w3c.dom.Node#getNamespaceURI()
     */
    public final String getNamespaceURI() {
        return null;
    }

    /**
     * Retrieves the namespace prefix of this node - will always return null since namespace support is not implemented
     * @return null value
     * @see org.w3c.dom.Node#getPrefix()
     */
    public final String getPrefix() {
        return null;
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param newChild The node to insert - ignored by this implementation
     * @param refChild The reference node, i.e., the node before which the new node must be inserted
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Node#insertBefore(org.w3c.dom.Node, org.w3c.dom.Node)
     */
    public final Node insertBefore(Node newChild, Node refChild) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * Returns always false since no special features are implemented
     * @param feature The name of the feature to test
     * @param version This is the version number of the feature to test
     * @return Always false
     * @see org.w3c.dom.Node#isSupported(java.lang.String, java.lang.String)
     */
    public final boolean isSupported(String feature, String version) {
        return false;
    }

    /**
     * Normalize the node - calls to this method will be ignored
     * @see org.w3c.dom.Node#normalize()
     */
    public final void normalize() {
        // not implemented
    }

    /**
     * Retrieves the owner document of this element
     * @return Owned document of the element
     * @see org.w3c.dom.Node#getOwnerDocument()
     */
    public Document getOwnerDocument() {
        return owner;
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param node The node to remove - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Node#removeChild(org.w3c.dom.Node)
     */
    public final Node removeChild(Node node) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param newChild The node to insert - ignored by this implementation
     * @param oldChild The node to remove - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Node#replaceChild(org.w3c.dom.Node, org.w3c.dom.Node)
     */
    public final Node replaceChild(Node newChild, Node oldChild) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param value New value to set - ignored by this implementation
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Node#setNodeValue(java.lang.String)
     */
    public void setNodeValue(String value) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param prefix Prefix to set - ignored by this implementation
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Node#setPrefix(java.lang.String)
     */
    public final void setPrefix(String prefix) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * Helper function to retrieve the current bean element factory instance
     * @return Bean analyzer instance
     */
    protected BeanElementFactory getElementFactory() {
        return owner.elementFactory;
    }

    /**
     * Gets the previous sibling in element context
     * @return Previous sibling in element context of this node
     * @see org.w3c.dom.Node#getPreviousSibling()
     */
    public Node getPreviousSibling() {
        return previousSibling;
    }

    /**
     * Gets the next sibling in element context
     * @return Next sibling in element context of this node
     * @see org.w3c.dom.Node#getNextSibling()
     */
    public Node getNextSibling() {
        return nextSibling;
    }

    /**
     * Sets the next sibling when creating new elements
     * @param element Element to set as next sibling
     */
    protected void setNextSibling(BeanBaseNode element) {
        nextSibling = element;
    }

    /**
     * Sets the previous sibling when creating new elements
     * @param element Element to set as next sibling
     */
    protected void setPreviousSibling(BeanBaseNode element) {
        previousSibling = element;
    }

    /**
     * Retrieves the parent element of this element
     * @return Parent Element node
     * @see org.w3c.dom.Node#getParentNode()
     */
    public Node getParentNode() {
        return parent;
    }
}
