/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/src/net/beandocument/BeanDocumentBuilder.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-03-17 21:04:48 +0100 (Do, 17 Mrz 2005) $
** Revision .........: $LastChangedRevision: 36 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import org.w3c.dom.Document;

/**
 * Builds bean based documents.<br/>
 * This class is used as starting point when constructing new DOM documents from any Java bean. The following example
 * illustrates the use of the class and provides all system properties to the new document:<br/>
 * <pre>
 *     Object xmlSource = System.getProperties();
 *     try {
 *         BeanDocumentBuilder beanBuilder = new BeanDocumentBuilder();
 *         beanBuilder.setMaxDepth(2);
 *         Document xml = beanBuilder.getDocument(xmlSource);
 *         TransformerFactory tf = TransformerFactory.newInstance();
 *         Transformer t = tf.newTransformer();
 *         t.transform(new DOMSource(xml), new StreamResult(System.out));
 *     } catch (Exception e) {
 *         System.err.println("Error during transformation: " + e);
 *         e.printStackTrace();
 *     }
 * </pre>
 * The result document as serialized XML looks like:
 * <pre>
 * &lt;?xml version="1.0" encoding="UTF-8"?&gt;
 * &lt;Properties className="java.util.Properties"&gt;
 *     &lt;propertyNames className="java.util.Hashtable$Enumerator"&gt;
 *         &lt;item className="java.lang.String"&gt;java.runtime.name&lt;/item&gt;
 *     &lt;/propertyNames&gt;
 *     &lt;hashCode className="java.lang.Integer"&gt;1436952335&lt;/hashCode&gt;
 *     &lt;toString className="java.lang.String"&gt;{java.runtime.name=Java(TM) 2 Runtime Environment (...)&lt;/toString&gt;
 *     &lt;size className="java.lang.Integer"&gt;54&lt;/size&gt;
 *     &lt;values className="java.util.Collections$SynchronizedCollection"&gt;
 *         &lt;item className="java.lang.String"&gt;Java(TM) 2 Runtime Environment, Standard Edition&lt;/item&gt;
 *     &lt;/values&gt;
 *     &lt;elements className="java.util.Hashtable$Enumerator"&gt;
 *         &lt;item className="java.lang.String"&gt;Java(TM) 2 Runtime Environment, Standard Edition&lt;/item&gt;
 *     &lt;/elements&gt;
 *     &lt;keys className="java.util.Hashtable$Enumerator"&gt;
 *         &lt;item className="java.lang.String"&gt;java.runtime.name&lt;/item&gt;
 *     &lt;/keys&gt;
 *     &lt;entrySet className="java.util.Collections$SynchronizedSet"&gt;
 *         &lt;item className="java.util.Hashtable$Entry"/&gt;
 *     &lt;/entrySet&gt;
 *     &lt;isEmpty className="java.lang.Boolean"&gt;false&lt;/isEmpty&gt;
 *     &lt;keySet className="java.util.Collections$SynchronizedSet"&gt;
 *         &lt;item className="java.lang.String"&gt;java.runtime.name&lt;/item&gt;
 *     &lt;/keySet&gt;
 *     &lt;getClass className="java.lang.Class"/&gt;
 *     &lt;element key="java.runtime.name" className="java.lang.String"&gt;Java(TM) 2 Runtime Environment, Standard Edition&lt;/element&gt;
 *     &lt;element key="vendor-url" className="java.lang.String"&gt;http://xml.apache.org/xalan-j&lt;/element&gt;
 *     &lt;element key="sun.boot.library.path" className="java.lang.String"&gt;C:\Program Files\Java\j2re1.4.2_05\bin&lt;/element&gt;
 *     &lt;element key="java.vm.version" className="java.lang.String"&gt;1.4.2_05-b04&lt;/element&gt;
 *     (...)
 * &lt;/Properties&gt;
 * </pre>
 * Additional mechanisms can be used to influence the behavior of the created documents:
 *  <ul>
 *      <li>Maximum parse depth: When using DOM documents for XSLT transformation, the DOM tree is fully parsed
 *          by the current Java 1.4.x parser. Since Java objects itself do not present a tree, the parsing would be
 *          infinite. Therefore the maximum parsing depth is set to {@link #MAX_DEPTH_DEFAULT}. To override this
 *          setting, call the method <code>{@link #setMaxDepth(int)}</code>.<br/>NOTE: Using this approach
 *          consumes a lot overhead for transformation and is not encouraged to be used!</li>
 *      <li>Certain bean content is filtered by default, if tis should be overridden, you can add in-and exclude filter
 *          sets using the <code>{@link #addIncludeFilter(BeanContentFilter)}</code> and
 *          <code>{@link #addExcludeFilter(BeanContentFilter)}</code></li>
 *      <li>Elements are handled depending on type. If a special handler should be used to process certain content
 *          use the <code>{@link #addElementHandler(Class, Class)}</code> method to set a custom handler.
 *          This handler class will be instantiated for the defined type. Currently basic handlers for 
 *          Number, String, Collections and Maps are implemented.</li>
 *  </ul>
 * @author Jens Scheffler
 */
public class BeanDocumentBuilder {
    
    /** default depth for element generation of bean documents */
    public static final int MAX_DEPTH_DEFAULT = 3;

    /** Instance for the bean element factory */
    private final BeanElementFactory elementFactory;
    
    /**
     * Creates a new DocumentBuilder instance
     */
    public BeanDocumentBuilder() {
        super();
        
        elementFactory = new BeanElementFactory();
        elementFactory.setMaxDepth(MAX_DEPTH_DEFAULT);
    }

    /**
     * Generates and retrieves a new DOM document instance from the provided bean
     * @param bean Object from which the document should be created
     * @return DOM Document instance of the bean
     * @throws IllegalArgumentException If bean is null
     */
    public Document getDocument(Object bean) {
        if (bean == null) {
            throw new IllegalArgumentException("bean must not be null!");
        }
        
        BeanDocument result = new BeanDocument(elementFactory, bean);
        elementFactory.setOwner(result);
        return result;
    }
    
    /**
     * Adds a special element handler class to the element factory
     * @param type Class type which need to match to use the specified handler
     * @param handler Handler class which should handle the element. This needs to be of type BeanElement
     */
    public void addElementHandler(Class type, Class handler) {
        elementFactory.addElementMapping(type, handler);
    }
    
    /**
     * Sets the maximum evaluation depth for bean documents.
     * If this option is not set, the default depth of MAX_DEPTH_DEFAULT will be applied for each document which
     * is going to be generated.
     * This setting is required to prevent infinite recursion in use with transformer engines.
     * @param max Maximum element depth to generate.
     * @see #MAX_DEPTH_DEFAULT
     */
    public void setMaxDepth(int max) {
        elementFactory.setMaxDepth(max);
    }
    
    /**
     * Adds a filter for evaluating bean element content.
     * Only elements which match to the given filter criteria will be visible from the created document
     * @param filter Filter criteria to add
     */
    public void addIncludeFilter(BeanContentFilter filter) {
        elementFactory.addIncludeFilter(filter);
    }

    /**
     * Adds a filter for evaluating bean element content.
     * Only elements which do not match to the given filter criteria will be visible from the created document
     * @param filter Filter criteria to add
     */
    public void addExcludeFilter(BeanContentFilter filter) {
        elementFactory.addExcludeFilter(filter);
    }
}
