/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/src/net/beandocument/BeanNamedNodeMap.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-02-27 15:11:14 +0100 (So, 27 Feb 2005) $
** Revision .........: $LastChangedRevision: 11 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.w3c.dom.DOMException;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * Map implementation of NamedNodeMap for bean documents
 * @author Jens Scheffler
 */
public class BeanNamedNodeMap implements NamedNodeMap {

    /** Map of all added nodes */
    private Map nodes;
    
    /**
     * Default Constructor for a new Named Node Map
     */
    protected BeanNamedNodeMap() {
        super();
        
        nodes = new HashMap();
    }

    /**
     * Adds a node to the named node map
     * @param name Name of the node to reference
     * @param node Node instance to add
     */
    protected void add(String name, Node node) {
        nodes.put(name, node);
    }

    /**
     * Retrieves the length of the nodes in the map
     * @return Number of items in the node map
     * @see org.w3c.dom.NamedNodeMap#getLength()
     */
    public int getLength() {
        return nodes.size();
    }

    /**
     * Retrieves the named item from the map.
     * @param name Name if the item to retrieve
     * @return Item object node or null if not existing
     * @see org.w3c.dom.NamedNodeMap#getNamedItem(java.lang.String)
     */
    public Node getNamedItem(String name) {
        return (Node) nodes.get(name);
    }

    /**
     * Retrieves the named item from the map.
     * Since this implementation does not support namespace, null will be returned if namespace specified.
     * @param namespace Namespace to use
     * @param name Name if the item to retrieve
     * @return Item object node or null if not existing
     * @see org.w3c.dom.NamedNodeMap#getNamedItemNS(java.lang.String, java.lang.String)
     */
    public Node getNamedItemNS(String namespace, String name) {
        if (namespace == null) {
            return getNamedItem(name);
        } 
        return null;
    }

    /**
     * Retrieves the n'th element from the map
     * @param index Item number to retrieve
     * @return n'th element of the map
     * @see org.w3c.dom.NamedNodeMap#item(int)
     */
    public Node item(int index) {
        // check for validity
        if (index >= nodes.size() || index < 0) {
            throw new ArrayIndexOutOfBoundsException("Invalid index of " + index + " specified, " + nodes.size() +
                    " nodes in map!");
        }

        Iterator i = nodes.values().iterator();
        Node result = null;
        while (i.hasNext() && index > -1) {
            result = (Node) i.next();
            index--;
        }
        return result;
    }

    /**
     * Removes the named items from the map.
     * @param name Name of the item to remove
     * @return The node which was just removed or null if not existing
     * @throws org.w3c.dom.DOMException (never)
     * @see org.w3c.dom.NamedNodeMap#removeNamedItem(java.lang.String)
     */
    public Node removeNamedItem(String name) throws DOMException {
        return (Node) nodes.remove(name);
    }

    /**
     * Removes the named items from the map.
     * @param namespace Namespace to use, only null is supported
     * @param name Name of the item to remove
     * @return The node which was just removed or null if not existing
     * @throws org.w3c.dom.DOMException (never)
     * @see org.w3c.dom.NamedNodeMap#removeNamedItemNS(java.lang.String, java.lang.String)
     */
    public Node removeNamedItemNS(String namespace, String name) throws DOMException {
        if (namespace != null) {
            return null;
        }
        return removeNamedItem(name);
    }

    /**
     * Sets a named item in the map.
     * @param item Item to set
     * @return The node which was just replaced or null if not existing so far
     * @throws org.w3c.dom.DOMException (never)
     * @see org.w3c.dom.NamedNodeMap#setNamedItem(org.w3c.dom.Node)
     */
    public Node setNamedItem(Node item) throws DOMException {
        return (Node) nodes.put(item.getNodeName(), item);
    }

    /**
     * Sets a named item in the map.
     * Calling this method will always throw an exception since the map is unmodifiable!
     * @param item Item to set - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.NamedNodeMap#setNamedItemNS(org.w3c.dom.Node)
     */
    public Node setNamedItemNS(Node item) throws DOMException {
        return setNamedItem(item);
    }
}
