/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/src/net/beandocument/BeanSimpleTypeElement.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-02-27 15:11:14 +0100 (So, 27 Feb 2005) $
** Revision .........: $LastChangedRevision: 11 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import org.w3c.dom.DOMException;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 * Element implementation for a simple type.
 * Simple types are:
 * <ul>
 *  <li>Long, Integer, Short (basically -> Number)</li>
 *  <li>Double, Float</li>
 *  <li>Byte, Character, Byte[], Character[]</li>
 *  <li>String, StringBuffer</li>
 * </ul>
 * @author Jens Scheffler
 */
public class BeanSimpleTypeElement extends BeanElement {
    
    /**
     * Implementation class for the text node content of the simple value bean
     * @author jscheffl
     */
    private class BeanTextNode extends BeanBaseNode implements Text {

        /** Text value of this node */
        private final String text;

        /**
         * Creates a new text node for the simple tag implementation
         * @param owner Owner document of this node
         * @param parent Parent node where this text belongs to
         * @param text Text to assign as value
         */
        public BeanTextNode(BeanDocument owner, BeanBaseNode parent, String text) {
            super(owner, parent);

            this.text = text;
        }

        /**
         * NOTE: This function is not implemented and therefore will just throw an exception!
         * @param pos Position where to split - ignored by this implementation
         * @return (This function will never return normally)
         * @throws org.w3c.dom.DOMException If method is called
         * @see org.w3c.dom.Text#splitText(int)
         */
        public Text splitText(int pos) throws DOMException {
            throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
        }

        /**
         * Retrieves the text length
         * @return Length of the text of this node
         * @see org.w3c.dom.CharacterData#getLength()
         */
        public int getLength() {
            return text.length();
        }

        /**
         * NOTE: This function is not implemented and therefore will just throw an exception!
         * @param offset Position where to start deletion - ignored by this implementation
         * @param count Length for deletion - ignored by this implementation
         * @throws org.w3c.dom.DOMException If method is called
         * @see org.w3c.dom.CharacterData#deleteData(int, int)
         */
        public void deleteData(int offset, int count) throws DOMException {
            throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
        }

        /**
         * Retrieves the text content of the node
         * @return Text content
         * @throws DOMException (never thrown)
         * @see org.w3c.dom.CharacterData#getData()
         */
        public String getData() throws DOMException {
            return getNodeValue();
        }

        /**
         * Retrieves a substring of the complete element
         * @param offset Start of string to extract
         * @param count Length of string to extract
         * @return Substring of the complete text content
         * @throws DOMException If the given parameters are invalid
         * @see org.w3c.dom.CharacterData#substringData(int, int)
         */
        public String substringData(int offset, int count) throws DOMException {
            try {
                return text.substring(offset, offset + count);
            } catch (IndexOutOfBoundsException e) {
                throw new DOMException(DOMException.INDEX_SIZE_ERR, e.getMessage());
            }
        }

        /**
         * NOTE: This function is not implemented and therefore will just throw an exception!
         * @param offset Offset where to start cutting - ignored by this implementation
         * @param count Length to replace - ignored by this implementation
         * @param replacement Replacement value to insert - ignored by this implementation
         * @throws org.w3c.dom.DOMException If method is called
         * @see org.w3c.dom.CharacterData#replaceData(int, int, java.lang.String)
         */
        public void replaceData(int offset, int count, String replacement) throws DOMException {
            throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
        }

        /**
         * NOTE: This function is not implemented and therefore will just throw an exception!
         * @param offset Offset where to start cutting - ignored by this implementation
         * @param content Content to insert - ignored by this implementation
         * @throws org.w3c.dom.DOMException If method is called
         * @see org.w3c.dom.CharacterData#insertData(int, java.lang.String)
         */
        public void insertData(int offset, String content) throws DOMException {
            throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
        }

        /**
         * NOTE: This function is not implemented and therefore will just throw an exception!
         * @param content Content to insert - ignored by this implementation
         * @throws org.w3c.dom.DOMException If method is called
         * @see org.w3c.dom.CharacterData#appendData(java.lang.String)
         */
        public void appendData(String content) throws DOMException {
            throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
        }

        /**
         * NOTE: This function is not implemented and therefore will just throw an exception!
         * @param content Content to insert - ignored by this implementation
         * @throws org.w3c.dom.DOMException If method is called
         * @see org.w3c.dom.CharacterData#setData(java.lang.String)
         */
        public void setData(String content) throws DOMException {
            throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
        }

        /**
         * Retrieves the node type for the text node
         * @return fixed constant value of Node.TEXT_NODE
         * @see org.w3c.dom.Node#getNodeType()
         */
        public short getNodeType() {
            return Node.TEXT_NODE;
        }

        /**
         * Returns always false since no attributes are applicable for a text node
         * @return false value
         * @see org.w3c.dom.Node#hasAttributes()
         */
        public boolean hasAttributes() {
            return false;
        }

        /**
         * Returns always false since no child nodes are applicable for a text node
         * @return false value
         * @see org.w3c.dom.Node#hasChildNodes()
         */
        public boolean hasChildNodes() {
            return false;
        }

        /**
         * Retrieves the local node name of this text node which corresponds to a fixed value.
         * @return Local node name, which corresponds with the full node name
         * @see org.w3c.dom.Node#getLocalName()
         */
        public String getLocalName() {
            return getNodeName();
        }

        /**
         * Retrieves the node name of this text node which corresponds to a fixed value.
         * @return "#text" as fixed value
         * @see org.w3c.dom.Node#getNodeName()
         */
        public String getNodeName() {
            return "#text";
        }

        /**
         * Retrieves the text of this text node
         * @return Text of this text node
         * @throws DOMException
         * @see org.w3c.dom.Node#getNodeValue()
         */
        public String getNodeValue() throws DOMException {
            return text;
        }

        /**
         * Retrieves the attributes of the node. Since the text node itself has no attributes, returns null.
         * @return null value
         * @see org.w3c.dom.Node#getAttributes()
         */
        public NamedNodeMap getAttributes() {
            return null;
        }

        /**
         * Retrieves the first child, which is (of course) not existing here.
         * @return null value
         * @see org.w3c.dom.Node#getFirstChild()
         */
        public Node getFirstChild() {
            return null;
        }

        /**
         * Retrieves the last child, which is (of course) not existing here.
         * @return null value
         * @see org.w3c.dom.Node#getLastChild()
         */
        public Node getLastChild() {
            return null;
        }

        /**
         * Retrieves the list of child nodes, which is an empty list in this case.
         * @return empty list of nodes
         * @see org.w3c.dom.Node#getChildNodes()
         */
        public NodeList getChildNodes() {
            return new BeanNodeList();
        }
    }

    /** value of this element */
    private final Object bean;
    
    /** caches list of child nodes of this simple element */
    private BeanNodeList childs;

    /**
     * Constructor for a new simple type bean element
     * @param owner Owner document of the element
     * @param parent parent node of this element
     * @param tagName Name for the tag of this element
     * @param bean bean which keeps the value of the tag
     */
    public BeanSimpleTypeElement(BeanDocument owner, BeanBaseNode parent, String tagName, Object bean) {
        super(owner, parent, tagName, bean);

        this.bean = bean;
    }

    /**
     * Retrieves the child nodes of this element.
     * The simple bean implementation will wrap the content into a text node and return this as single child
     * @return List of child node
     * @see org.w3c.dom.Node#getChildNodes()
     */
    public NodeList getChildNodes() {
        if (childs == null) {
            childs = new BeanNodeList();
            childs.add(new BeanTextNode(owner, this, getText()));
        }

        return childs;
    }

    /**
     * Generates the text corresponding to the element content bean
     * @return Text of this text node
     */
    private String getText() {
        if (bean == null) {
            return "";
        } else if (bean instanceof String) {
            return (String) bean;
        } else if (bean instanceof Byte) {
            byte[] temp = new byte[1];
            temp[0] = ((Byte) bean).byteValue();
            return new String(temp);
        } else if (bean instanceof Byte[]) {
            Byte[] value = (Byte[]) bean;
            byte[] temp = new byte[value.length];
            for (int i = 0; i < value.length; i++) {
                temp[i] = value[i].byteValue();
            }
            return new String(temp);
        } else if (bean instanceof Character[]) {
            Character[] value = (Character[]) bean;
            char[] temp = new char[value.length];
            for (int i = 0; i < value.length; i++) {
                temp[i] = value[i].charValue();
            }
            return new String(temp);
        }
        return bean.toString();
    }
}
