/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/src/net/beandocument/SimpleBeanContentFilter.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-03-25 14:52:56 +0100 (Fr, 25 Mrz 2005) $
** Revision .........: $LastChangedRevision: 46 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

/**
 * Simple filter implementation which matches the given content as regular expressions against the given conditions.
 * @author Jens Scheffler
 */
public class SimpleBeanContentFilter implements BeanContentFilter {
    
    /** Filter expression for the class name to match for */
    private final String beanClassNameFilter;

    /** Filter class for the bean class to verify */
    private final Class beanClassFilter;
    
    /** Filter expression for the member name to filter for */
    private final String memberNameFilter;
    
    /** Filter expression for the members class name to filter for */
    private final String memberClassNameFilter;
    
    /** Filter class for the member class to verify */
    private final Class memberClassFilter;

    /**
     * Constructor for a new simple filter.
     * All given conditions need to be set with this constructor. Each condition will be matched using regular
     * expressions against the test criterias. Criterias which are supplied as <code>null</code> will be ignored.
     * @param beanClassNameFilter Filter expression for the class name to match for
     * @param memberNameFilter Filter expression for the member name to filter for
     * @param memberClassNameFilter Filter expression for the members class name to filter for
     */
    public SimpleBeanContentFilter(String beanClassNameFilter, String memberNameFilter, String memberClassNameFilter) {
                
        this.beanClassNameFilter = beanClassNameFilter;
        this.beanClassFilter = null;
        this.memberNameFilter = memberNameFilter;
        this.memberClassNameFilter = memberClassNameFilter;
        this.memberClassFilter = null;
    }
    
    /**
     * Constructor for a new simple filter.
     * All given conditions need to be set with this constructor. String conditions will be matched using regular
     * expressions against the test criterias. Class type criterias will be matched using the check if assignable. 
     * Criterias which are supplied as <code>null</code> will be ignored.
     * @param beanClassFilter Class which needs to correspond with the given filter criteria
     * @param memberNameFilter Filter expression for the member name to filter for
     * @param memberClassNameFilter Filter expression for the members class name to filter for
     */
    public SimpleBeanContentFilter(Class beanClassFilter, String memberNameFilter, String memberClassNameFilter) {
        
        this.beanClassNameFilter = null;
        this.beanClassFilter = beanClassFilter;
        this.memberNameFilter = memberNameFilter;
        this.memberClassNameFilter = memberClassNameFilter;
        this.memberClassFilter = null;
    }
    
    /**
     * Constructor for a new simple filter.
     * All given conditions need to be set with this constructor. String conditions will be matched using regular
     * expressions against the test criterias. Class type criterias will be matched using the check if assignable. 
     * Criterias which are supplied as <code>null</code> will be ignored.
     * @param beanClassNameFilter Filter expression for the class name to match for
     * @param memberNameFilter Filter expression for the member name to filter for
     * @param memberClassFilter Class which needs to correspond with the given filter criteria
     */
    public SimpleBeanContentFilter(String beanClassNameFilter, String memberNameFilter, Class memberClassFilter) {
        
        this.beanClassNameFilter = beanClassNameFilter;
        this.beanClassFilter = null;
        this.memberNameFilter = memberNameFilter;
        this.memberClassNameFilter = null;
        this.memberClassFilter = memberClassFilter;
    }
    
    /**
     * Constructor for a new simple filter.
     * All given conditions need to be set with this constructor. String conditions will be matched using regular
     * expressions against the test criterias. Class type criterias will be matched using the check if assignable. 
     * Criterias which are supplied as <code>null</code> will be ignored.
     * @param beanClassFilter Class which needs to correspond with the given filter criteria
     * @param memberNameFilter Filter expression for the member name to filter for
     * @param memberClassFilter Class which needs to correspond with the given filter criteria
     */
    public SimpleBeanContentFilter(Class beanClassFilter, String memberNameFilter, Class memberClassFilter) {
        
        this.beanClassNameFilter = null;
        this.beanClassFilter = beanClassFilter;
        this.memberNameFilter = memberNameFilter;
        this.memberClassNameFilter = null;
        this.memberClassFilter = memberClassFilter;
    }

    /**
     * Tries to match a given member information against the filter instance. 
     * @param beanClass Class of the bean to match
     * @param memberName Name of the member which is evaluated
     * @param memberClass Class of the member
     * @return Flag if the given member information matches for the given filter
     * @see net.beandocument.BeanContentFilter#match(java.lang.Class, java.lang.String, java.lang.Class)
     */
    public boolean match(Class beanClass, String memberName, Class memberClass) {

        // match class name first
        if (beanClassNameFilter != null) {
            if (!beanClass.getName().matches(beanClassNameFilter)) {
                return false;
            }
        }
        
        // match class type
        if (beanClassFilter != null) {
            if (!beanClassFilter.isAssignableFrom(beanClass)) {
                return false;
            }
        }
        
        // match members name
        if (memberNameFilter != null) {
            if (!memberName.matches(memberNameFilter)) {
                return false;
            }
        }
        
        // match class name
        if (memberClassNameFilter != null) {
            if (!memberClass.getName().matches(memberClassNameFilter)) {
                return false;
            }
        }
        
        // match member type
        if (memberClassFilter != null) {
            if (!memberClassFilter.isAssignableFrom(memberClass)) {
                return false;
            }
        }
        
        // if execution has come here, this seems to match!
        return true;
    }
}
